@@ -11,5 +11,6 @@
 pub mod contest;
 pub mod problem;
 pub mod result;
+pub mod sty;
 pub mod template;
-pub mod utils;
+mod utils;
