@@ -1,4 +1,4 @@
-//! Parses and manipulates problem yaml
+//! Fetching and filtering functions for problems
 
 use crate::result::MapmErr;
 use crate::result::MapmErr::*;
@@ -18,9 +18,6 @@ use serde::{Deserialize, Serialize};
 
 use serde_yaml::Value;
 
-pub type Vars = HashMap<String, String>;
-pub type Solutions = Vec<HashMap<String, String>>;
-
 #[derive(Debug, Serialize, Deserialize)]
 struct SerializedProblem {
     pub solutions: Option<Solutions>,
@@ -28,8 +25,12 @@ struct SerializedProblem {
     pub vars: HashMap<String, Value>,
 }
 
+pub type Vars = HashMap<String, String>;
+pub type Solutions = Vec<HashMap<String, String>>;
+
 #[derive(Debug, PartialEq, Clone)]
 pub struct Problem {
+    pub name: String,
     pub vars: Vars,
     pub solutions: Solutions,
 }
@@ -66,15 +67,17 @@ pub enum Views {
     Hide(Vec<String>),
 }
 
-/// Gets a problem from passed in filepath and name of problem
+/// Fetches problem given the name (which corresponds to the filename) and the directory to find it
+/// in
 
-pub fn fetch_problem(problem_name: &str, problem_dir: &Path) -> MapmResult<Problem> {
-    let problem_src = &problem_dir.join(&[problem_name, ".yml"].concat());
-    match fs::read_to_string(problem_src) {
-        Ok(problem_yaml) => parse_problem_yaml(&problem_yaml),
+pub fn fetch_problem<T: AsRef<Path>>(problem_name: &str, problem_dir: T) -> MapmResult<Problem> {
+    let problem_dir: &Path = problem_dir.as_ref();
+    let problem_path = &problem_dir.join(&format!("{}.yml", problem_name));
+    match fs::read_to_string(problem_path) {
+        Ok(problem_yaml) => parse_problem_yaml(problem_name, &problem_yaml),
         Err(_) => Err(ProblemErr(format!(
             "Could not read problem `{}` from {:?}",
-            problem_name, problem_src
+            problem_name, problem_path
         ))),
     }
 }
@@ -91,69 +94,69 @@ impl Problem {
     /// use mapm::problem::Filter;
     /// use std::collections::HashMap;
     ///
-    /// let mut vars = HashMap::new();
+    /// let mut vars: HashMap<String, String> = HashMap::new();
     /// vars.insert(String::from("problem"), String::from("What is $1+1$?"));
     /// vars.insert(String::from("author"), String::from("Dennis Chen"));
     /// vars.insert(String::from("difficulty"), String::from("5"));
     ///
-    /// let mut solution_one = HashMap::new();
+    /// let mut solution_one: HashMap<String, String> = HashMap::new();
     /// solution_one.insert(String::from("text"), String::from("It's probably $2$."));
     /// solution_one.insert(String::from("author"), String::from("Dennis Chen"));
     /// let mut solution_two = HashMap::new();
     /// solution_two.insert(String::from("text"), String::from("The answer is $2$, but my proof is too small to fit into the margin."));
     /// solution_two.insert(String::from("author"), String::from("Pierre de Fermat"));
     ///
-    /// let mut solutions = Vec::from([solution_one, solution_two]);
+    /// let mut solutions: Vec<HashMap<String, String>> = vec![solution_one, solution_two];
     ///
-    /// let problem = Problem { vars, solutions };
+    /// let problem = Problem { name: String::from("problem"), vars, solutions };
     ///
-    /// assert_eq!(problem.try_filter(Filter::Exists{key: String::from("subject")}), false);
-    /// assert_eq!(problem.try_filter(Filter::Exists{key: String::from("solutions")}), true);
+    /// assert_eq!(problem.try_filter(&Filter::Exists{key: String::from("subject")}), false);
+    /// assert_eq!(problem.try_filter(&Filter::Exists{key: String::from("solutions")}), true);
     ///
-    /// assert_eq!(problem.try_filter(Filter::Gt{key: String::from("difficulty"), val: 5}), false);
-    /// assert_eq!(problem.try_filter(Filter::Ge{key: String::from("difficulty"), val: 5}), true);
-    /// assert_eq!(problem.try_filter(Filter::Lt{key: String::from("difficulty"), val: 5}), false);
-    /// assert_eq!(problem.try_filter(Filter::Le{key: String::from("difficulty"), val: 5}), true);
+    /// assert_eq!(problem.try_filter(&Filter::Gt{key: String::from("difficulty"), val: 5}), false);
+    /// assert_eq!(problem.try_filter(&Filter::Ge{key: String::from("difficulty"), val: 5}), true);
+    /// assert_eq!(problem.try_filter(&Filter::Lt{key: String::from("difficulty"), val: 5}), false);
+    /// assert_eq!(problem.try_filter(&Filter::Le{key: String::from("difficulty"), val: 5}), true);
     ///
     /// ```
 
-    pub fn try_filter(&self, filter: Filter) -> bool {
+    pub fn try_filter(&self, filter: &Filter) -> bool {
         match filter {
             Exists { key } => {
                 if key == "solutions" {
-                    self.solutions.len() > 0
+                    !self.solutions.is_empty()
                 } else {
-                    self.vars.contains_key(&key)
+                    self.vars.contains_key(key)
                 }
             }
-            Eq { key, val } => match self.vars.get(&key) {
-                Some(problem_value) => problem_value == &val,
+            Eq { key, val } => match self.vars.get(key) {
+                Some(problem_value) => problem_value == val,
                 None => false,
             },
-            Gt { key, val } => match self.vars.get(&key) {
+            Gt { key, val } => match self.vars.get(key) {
                 Some(problem_value) => match problem_value.parse::<u32>() {
-                    Ok(problem_value_u32) => problem_value_u32 > val,
+                    Ok(problem_value_u32) => problem_value_u32 > *val,
                     Err(_) => false,
                 },
                 None => false,
             },
-            Lt { key, val } => match self.vars.get(&key) {
+            Lt { key, val } => match self.vars.get(key) {
                 Some(problem_value) => match problem_value.parse::<u32>() {
-                    Ok(problem_value_u32) => problem_value_u32 < val,
+                    Ok(problem_value_u32) => problem_value_u32 < *val,
                     Err(_) => false,
                 },
                 None => false,
             },
-            Ge { key, val } => match self.vars.get(&key) {
+            Ge { key, val } => match self.vars.get(key) {
                 Some(problem_value) => match problem_value.parse::<u32>() {
-                    Ok(problem_value_u32) => problem_value_u32 >= val,
+                    Ok(problem_value_u32) => problem_value_u32 >= *val,
                     Err(_) => false,
                 },
                 None => false,
             },
-            Le { key, val } => match self.vars.get(&key) {
+            Le { key, val } => match self.vars.get(key) {
                 Some(problem_value) => match problem_value.parse::<u32>() {
-                    Ok(problem_value_u32) => problem_value_u32 <= val,
+                    Ok(problem_value_u32) => problem_value_u32 <= *val,
                     Err(_) => false,
                 },
                 None => false,
@@ -165,7 +168,7 @@ impl Problem {
     ///
     /// If `filters` is empty, then every problem will pass the filter.
 
-    pub fn filter(self, filters: Vec<FilterAction>) -> Option<Problem> {
+    pub fn filter(self, filters: &Vec<FilterAction>) -> Option<Problem> {
         for filter_action in filters {
             match filter_action {
                 Positive(filter) => {
@@ -191,34 +194,45 @@ impl Problem {
     /// use mapm::problem::Views;
     /// use mapm::problem::Views::{Show, Hide};
     /// use mapm::problem::Problem;
-    /// use mapm::problem::parse_problem_yaml;
     /// use mapm::problem::Filter;
     /// use std::collections::HashMap;
     ///
-    /// let problem_yaml = "problem: What is $1+1$?
-    /// author: Dennis Chen
-    /// answer: 2
-    /// solutions:
-    ///   - text: It's $2$.
-    ///     author: Alexander
-    /// ";
+    /// let problem = Problem {
+    ///     name: String::from("problem"),
+    ///     vars: HashMap::from([
+    ///         (String::from("problem"), String::from("What is $1+1$?")),
+    ///         (String::from("author"), String::from("Dennis Chen")),
+    ///         (String::from("answer"), String::from("2"))
+    ///     ]),
+    ///     solutions: vec![HashMap::from([
+    ///         (String::from("text"), String::from("It's $2$.")),
+    ///         (String::from("author"), String::from("Alexander")),
+    ///     ])]
+    /// };
     ///
-    /// let problem = parse_problem_yaml(problem_yaml).unwrap();
+    /// let show: Views = Show(vec![String::from("author"), String::from("answer"), String::from("solutions")]);
+    /// let show_filtered_problem = problem.clone().filter_keys(&show);
+    /// assert_eq!(show_filtered_problem.0, HashMap::from([
+    ///     (String::from("author"), String::from("Dennis Chen")),
+    ///     (String::from("answer"), String::from("2"))
+    /// ]));
+    /// assert_eq!(show_filtered_problem.1, Some(vec![
+    ///     HashMap::from([
+    ///         (String::from("text"), String::from("It's $2$.")),
+    ///         (String::from("author"), String::from("Alexander"))
+    ///     ])
+    /// ]));
     ///
-    /// let show: Views = Show(Vec::from([String::from("author"), String::from("answer"), String::from("solutions")]));
-    /// let show_filtered_problem = problem.clone().filter_keys(show);
-    /// assert_eq!(show_filtered_problem.0, HashMap::from([(String::from("author"), String::from("Dennis Chen")), (String::from("answer"), String::from("2"))]));
-    /// assert_eq!(show_filtered_problem.1, Some(Vec::from([
-    ///     HashMap::from([(String::from("text"), String::from("It's $2$.")), (String::from("author"), String::from("Alexander"))])
-    /// ])));
-    ///
-    /// let hide: Views = Hide(Vec::from([String::from("solutions"), String::from("author")]));
-    /// let hide_filtered_problem = problem.clone().filter_keys(hide);
-    /// assert_eq!(hide_filtered_problem.0, HashMap::from([(String::from("problem"), String::from("What is $1+1$?")), (String::from("answer"), String::from("2"))]));
+    /// let hide: Views = Hide(vec![String::from("solutions"), String::from("author")]);
+    /// let hide_filtered_problem = problem.clone().filter_keys(&hide);
+    /// assert_eq!(hide_filtered_problem.0, HashMap::from([
+    ///     (String::from("problem"), String::from("What is $1+1$?")),
+    ///     (String::from("answer"), String::from("2")),
+    /// ]));
     /// assert_eq!(hide_filtered_problem.1, None);
     /// ```
 
-    pub fn filter_keys(self, views: Views) -> (Vars, Option<Solutions>) {
+    pub fn filter_keys(self, views: &Views) -> (Vars, Option<Solutions>) {
         match views {
             Show(tags) => {
                 let mut vars: Vars = HashMap::new();
@@ -258,31 +272,32 @@ impl Problem {
     /// ## Expected success
     ///
     /// ```
-    /// use mapm::problem::*;
-    /// use mapm::template::*;
-    /// let problem_yaml = "problem: What is $1+1$?
-    /// solutions:
-    ///   - text: Some say the answer is $2$.
-    ///     author: Dennis Chen
-    /// ";
+    /// use mapm::problem::Problem;
+    /// use mapm::template::Template;
+    /// use std::collections::HashMap;
     ///
-    /// let template_yaml = "engine: pdflatex
-    /// texfiles:
-    ///   problems.tex: ${title}.PDF
-    ///   solutions.tex: ${title}-sols.PDF
-    /// problem_count: 1
-    /// vars:
-    ///   - title
-    ///   - year
-    /// problemvars:
-    ///   - problem
-    /// solutionvars:
-    ///   - text
-    ///   - author
-    /// ";
+    /// let problem = Problem {
+    ///     name: String::from("problem"),
+    ///     vars: HashMap::from([
+    ///         (String::from("problem"), String::from("What is $1+1?$"))
+    ///     ]),
+    ///     solutions: vec![HashMap::from([
+    ///         (String::from("text"), String::from("Some say the answer is $2$.")),
+    ///         (String::from("author"), String::from("Dennis Chen")),
+    ///     ])]
+    /// };
     ///
-    /// let problem = parse_problem_yaml(problem_yaml).unwrap();
-    /// let template = parse_template_yaml(template_yaml).unwrap();
+    /// let template = Template {
+    ///     name: String::from("template"),
+    ///     engine: String::from("pdflatex"),
+    ///     problem_count: Some(1),
+    ///     texfiles: HashMap::from([
+    ///         (String::from("problems.tex"), String::from("problems.pdf"))
+    ///     ]),
+    ///     vars: vec![String::from("title"), String::from("year")],
+    ///     problemvars: vec![String::from("problem")],
+    ///     solutionvars: vec![String::from("text"), String::from("author")],
+    /// };
     ///
     /// assert!(problem.check_template(&template).is_none());
     /// ```
@@ -290,35 +305,32 @@ impl Problem {
     /// ## Expected failure
     ///
     /// ```
-    /// use mapm::problem::*;
-    /// use mapm::template::*;
+    /// use mapm::problem::Problem;
+    /// use mapm::template::Template;
     /// use mapm::result::MapmErr;
     /// use mapm::result::MapmErr::*;
-    /// let problem_yaml = "problem: What is $1+1$?
-    /// solutions:
-    ///   - text: Some say the answer is $2$.
-    ///     author: Dennis Chen
-    /// ";
+    /// use std::collections::HashMap;
     ///
-    /// let template_yaml = "engine: pdflatex
-    /// texfiles:
-    ///   problems.tex: ${title}.PDF
-    ///   solutions.tex: ${title}-sols.PDF
-    /// vars:
-    ///   - title
-    ///   - year
-    /// problem_count: 1
-    /// problemvars:
-    ///   - problem
-    ///   - author
-    /// solutionvars:
-    ///   - text
-    ///   - author
-    /// ";
-    ///
-    /// let problem = parse_problem_yaml(problem_yaml).unwrap();
-    /// let template = parse_template_yaml(template_yaml).unwrap();
+    /// let problem = Problem {
+    ///     name: String::from("problem"),
+    ///     vars: HashMap::from([(String::from("problem"), String::from("What is $1+1?$"))]),
+    ///     solutions: vec![HashMap::from([
+    ///         (String::from("text"), String::from("Some say the answer is $2$.")),
+    ///         (String::from("author"), String::from("Dennis Chen")),
+    ///     ])]
+    /// };
     ///
+    /// let template = Template {
+    ///     name: String::from("template"),
+    ///     engine: String::from("pdflatex"),
+    ///     problem_count: Some(1),
+    ///     texfiles: HashMap::from([(
+    ///         String::from("problems.tex"), String::from("problems.pdf")
+    ///     )]),
+    ///     vars: vec!(String::from("title"), String::from("year")) ,
+    ///     problemvars: vec!(String::from("problem"), String::from("author")),
+    ///     solutionvars: vec!(String::from("text"), String::from("author")),
+    /// };
     /// let template_check = problem.check_template(&template).unwrap();
     ///
     /// assert_eq!(template_check.len(), 1);
@@ -337,9 +349,7 @@ impl Problem {
 
         for problemvar in &template.problemvars {
             if !self.vars.contains_key(problemvar) {
-                mapm_errs.push(ProblemErr(
-                    ["Does not contain key `", &problemvar, "`"].concat(),
-                ));
+                mapm_errs.push(ProblemErr(format!("Does not contain key `{}`", problemvar)));
             }
         }
         let mut index: u32 = 0;
@@ -347,62 +357,22 @@ impl Problem {
             index += 1;
             for solutionvar in &template.solutionvars {
                 if !map.contains_key(solutionvar) {
-                    mapm_errs.push(SolutionErr(
-                        [
-                            "Solution `",
-                            &index.to_string(),
-                            "` does not contain key `",
-                            &solutionvar,
-                            "`",
-                        ]
-                        .concat(),
-                    ));
+                    mapm_errs.push(SolutionErr(format!(
+                        "Solution `{}` does not contain key `{}`",
+                        index, solutionvar,
+                    )));
                 }
             }
         }
-        if mapm_errs.len() > 0 {
-            return Some(mapm_errs);
+        if !mapm_errs.is_empty() {
+            Some(mapm_errs)
         } else {
-            return None;
+            None
         }
     }
 }
 
-///
-/// # Usage
-///
-/// ```
-/// use mapm::problem::Problem;
-/// use mapm::problem::parse_problem_yaml;
-/// use std::collections::HashMap;
-/// let yaml = "problem: What is $1+1$?
-/// author: Dennis Chen
-/// solutions:
-///   - text: It's probably $2$.
-///     author: Dennis Chen
-///   - text: The answer is $2$, but my proof is too small to fit into the margin.
-///     author: Pierre de Fermat
-/// ";
-/// let problem = parse_problem_yaml(yaml).unwrap();
-///
-/// let mut vars = HashMap::new();
-/// vars.insert(String::from("problem"), String::from("What is $1+1$?"));
-/// vars.insert(String::from("author"), String::from("Dennis Chen"));
-///
-/// let mut solution_one = HashMap::new();
-/// solution_one.insert(String::from("text"), String::from("It's probably $2$."));
-/// solution_one.insert(String::from("author"), String::from("Dennis Chen"));
-/// let mut solution_two = HashMap::new();
-/// solution_two.insert(String::from("text"), String::from("The answer is $2$, but my proof is too small to fit into the margin."));
-/// solution_two.insert(String::from("author"), String::from("Pierre de Fermat"));
-///
-/// let mut solutions = Vec::from([solution_one, solution_two]);
-///
-/// assert_eq!(problem.vars, vars);
-/// assert_eq!(problem.solutions, solutions);
-/// ```
-
-pub fn parse_problem_yaml(yaml: &str) -> MapmResult<Problem> {
+fn parse_problem_yaml(name: &str, yaml: &str) -> MapmResult<Problem> {
     match serde_yaml::from_str::<SerializedProblem>(yaml) {
         Ok(problem) => {
             let mut vars: Vars = HashMap::new();
@@ -421,7 +391,7 @@ pub fn parse_problem_yaml(yaml: &str) -> MapmResult<Problem> {
                         };
                     }
                     _ => {
-                        return Err(ProblemErr(["Could not parse key `", &key, "`"].concat()));
+                        return Err(ProblemErr(format!("Could not parse key `{}`", key)));
                     }
                 }
             }
@@ -431,11 +401,13 @@ pub fn parse_problem_yaml(yaml: &str) -> MapmResult<Problem> {
                 None => Vec::new(),
             };
 
-            return Ok(Problem { vars, solutions });
-        }
-        Err(err) => {
-            return Err(ProblemErr(err.to_string()));
+            Ok(Problem {
+                name: String::from(name),
+                vars,
+                solutions,
+            })
         }
+        Err(err) => Err(ProblemErr(err.to_string())),
     }
 }
 
@@ -445,42 +417,81 @@ pub fn parse_problem_yaml(yaml: &str) -> MapmResult<Problem> {
 ///
 /// Returns a header TeX string as well (for contest compilation if necessary)
 
-pub(crate) fn write_as_tex(problem_vec: Vec<Problem>) -> String {
+pub(crate) fn write_as_tex(problem_vec: &Vec<Problem>) -> String {
     let mut problem_number = 0;
     let mut headers = String::new();
 
+    headers.push_str(&format!(
+        "\\expandafter\\def\\csname mapm@probcount\\endcsname{{{}}}",
+        problem_vec.len()
+    ));
+
     for problem in problem_vec {
         problem_number += 1;
-        headers.push_str(
-            &[
-                "\\expandafter\\def\\csname mapm@solcount@",
-                &problem_number.to_string(),
-                "\\endcsname{",
-                &problem.solutions.len().to_string(),
-                "}",
-            ]
-            .concat(),
-        );
+        headers.push_str(&format!(
+            "\\expandafter\\def\\csname mapm@solcount@{}\\endcsname{{{}}}",
+            problem_number,
+            problem.solutions.len()
+        ));
+        headers.push_str(&format!(
+            "\\expandafter\\def\\csname mapm@probname@{}\\endcsname{{{}}}",
+            problem_number, problem.name
+        ));
         for (key, val) in &problem.vars {
-            let filename = &["mapm-prob-", &problem_number.to_string(), "-", key, ".tex"].concat();
-            fs::write(filename, val).expect(&["Could not write to `", &filename, "`"].concat());
+            let filename = &format!("mapm-prob-{}-{}.tex", problem_number, key);
+            fs::write(filename, val)
+                .unwrap_or_else(|_| panic!("Could not write to `{}`", filename));
         }
         for (pos, map) in problem.solutions.iter().enumerate() {
             for (key, val) in map {
-                let solution_number = &pos + 1;
-                let filename = &[
-                    "mapm-sol-",
-                    &problem_number.to_string(),
-                    "-",
-                    &solution_number.to_string(),
-                    "-",
-                    key,
-                    ".tex",
-                ]
-                .concat();
-                fs::write(filename, val).expect(&["Could not write to `", &filename, "`"].concat());
+                let solution_number = pos + 1;
+                let filename = &format!(
+                    "mapm-sol-{}-{}-{}.tex",
+                    problem_number, solution_number, key,
+                );
+                fs::write(filename, val)
+                    .unwrap_or_else(|_| panic!("Could not write to `{}`", filename));
             }
         }
     }
     headers
 }
+
+#[cfg(test)]
+mod tests {
+    #[test]
+    fn test_parse() {
+        use super::parse_problem_yaml;
+        use std::collections::HashMap;
+
+        let yaml = "problem: What is $1+1$?
+author: Dennis Chen
+solutions:
+  - text: It's probably $2$.
+    author: Dennis Chen
+  - text: The answer is $2$, but my proof is too small to fit into the margin.
+    author: Pierre de Fermat
+";
+        let problem = parse_problem_yaml("problem", yaml).unwrap();
+
+        let mut vars = HashMap::new();
+        vars.insert(String::from("problem"), String::from("What is $1+1$?"));
+        vars.insert(String::from("author"), String::from("Dennis Chen"));
+
+        let mut solution_one = HashMap::new();
+        solution_one.insert(String::from("text"), String::from("It's probably $2$."));
+        solution_one.insert(String::from("author"), String::from("Dennis Chen"));
+        let mut solution_two = HashMap::new();
+        solution_two.insert(
+            String::from("text"),
+            String::from("The answer is $2$, but my proof is too small to fit into the margin."),
+        );
+        solution_two.insert(String::from("author"), String::from("Pierre de Fermat"));
+
+        let solutions = vec![solution_one, solution_two];
+
+        assert_eq!(problem.name, "problem");
+        assert_eq!(problem.vars, vars);
+        assert_eq!(problem.solutions, solutions);
+    }
+}
